function BellCurve(t)
	return math.exp(-t*t)
end

function FreqToColor(t)
	return {BellCurve((t-0.25)*6)+0.001,BellCurve((t-0.4)*6)+0.01,BellCurve((t-0.75)*6)}
end
--[[
function FreqToColor(t)
	return {BellCurve((t-0.75)*6),BellCurve((t-0.25)*6)+0.001,BellCurve((t-0.4)*6)+0.01}
end
]]

--- we have 48 notes per octave (4 notes per half-tone)
--- notes start 5 octaves below A440
local notes_per_octave=48

local max_freq=notes_per_octave*8

local rgb_t={0,0,0}
local sound_color={0,0,0}

local sound_color_raw={0,0,0}
local sound_color_avgv=0.01
local sound_color_avg_time=20
local sound_color_avg_add=0.2
--local sound_color_avgv=0.3
--local sound_color_avg_time=1E20

local mvis_brightness=0
local mvis_distortion=0
local mvis_filter_melody=0.5


notes_processed={}

--- Here we override main.lua 's on_audio_frame. See data/main.lua for details
local main_on_audio_frame=on_audio_frame
function on_audio_frame(dt) --- called by engine
	main_on_audio_frame(dt)
	if notes==nil then return end

	notes_processed={}
	notes_sum={}

	sound_color_raw={0,0,0}
	local sum=0

	for i,v in ipairs(notes) do
		sum=sum+v
		notes_sum[i]=sum
	end

	for i,v in ipairs(notes) do
	end

	local window_r=notes_per_octave/6

	local mvis_filter_melody_b=1-mvis_filter_melody
	for i,v in ipairs(notes) do
		local a=(v-1.5*((notes_sum[i+window_r] or sum)-(notes_sum[i-window_r] or 0))*0.5/window_r)
		if a<0 then
			a=0
		end

		local q=(a*mvis_filter_melody + notes[i]*mvis_filter_melody_b)

		notes_processed[i]=q



		local f=i/max_freq
		local c=FreqToColor(f)

		q=q*2

		sound_color_raw[1]=sound_color_raw[1]+c[1]*q
		sound_color_raw[2]=sound_color_raw[2]+c[2]*q
		sound_color_raw[3]=sound_color_raw[3]+c[3]*q
	end
	local color_v=(sound_color_raw[1]+sound_color_raw[2]+sound_color_raw[3])/3
	sound_color_avgv=sound_color_avgv*(1.0-dt/sound_color_avg_time) + color_v*(dt/sound_color_avg_time);
	local a=1.0/(sound_color_avgv+sound_color_avg_add)
	sound_color={sound_color_raw[1]*a, sound_color_raw[2]*a, sound_color_raw[3]*a }
	local maxv=math.max(unpack(sound_color))
	if maxv>1 then
	  sound_color_avgv=sound_color_avgv*maxv
	end

	--LogMessage("sound color: ",table.tostring(sound_color))


	rgb_t[1]=rgb_t[1]+sound_color[1]*dt*mvis_distortion
	rgb_t[2]=rgb_t[2]+sound_color[2]*dt*mvis_distortion
	rgb_t[3]=rgb_t[3]+sound_color[3]*dt*mvis_distortion



end


function DrawHUDFunc.DrawMusicVisualizerDebug(width,height,bloom)
	if bloom then return false end

	if GetOption("devel/draw_debug_info")~="1" then return false end--- dont want this function now.

	if not notes then return end
	gl.Enable(gl.BLEND)
	gl.Begin(gl.QUADS);
	local piano_keyboard_height=20

	local black={false,true,false,true,false,false,true,false,true,false,true,false}

	local halftones=#notes*12/notes.steps_per_octave
	local halftone_width=notes.steps_per_octave/12

	local start_x=width-#notes

	local spectra_y=height
	local spectra_height=60
	local chart_y=height-piano_keyboard_height-spectra_height
	local piano_keyboard_y=height-spectra_height

	local function DrawBar(i,a)
		local x=i

		local ta=(math.log(a*1E5))*0.05
		if ta>0 then
			--- vertices of a rectangle are specified like this
			gl.Vertex(x+width-#notes, chart_y)
			gl.Vertex(x+width-#notes +1, chart_y)
			gl.Vertex(x+width-#notes +1, chart_y-ta*height)
			gl.Vertex(x+width-#notes, chart_y-ta*height)
		end
	end

	for i=2,#notes-1 do

		--local is_peak=a>0.52*(smooth_notes[i-1]+smooth_notes[i+1]) and a>smooth_avg_amp
		local x=i

		local f=i/max_freq
		local c=FreqToColor(f)

		local a=notes[i]
		gl.Color(0,1,0)
		DrawBar(i,a)

		a=notes_processed[i]
		gl.Color(1,0,0)
		DrawBar(i,a)

		a=a*8

		local c2={c[1]*a,c[2]*a,c[3]*a}

		gl.Color(unpack(c))
		gl.Vertex(x+width-#notes, spectra_y)
		gl.Vertex(x+width-#notes +1, spectra_y)
		gl.Color(unpack(c2))
		gl.Vertex(x+width-#notes +1, spectra_y-spectra_height*0.5)
		gl.Vertex(x+width-#notes, spectra_y-spectra_height*0.5)

		gl.Vertex(x+width-#notes, spectra_y-spectra_height*0.5)
		gl.Vertex(x+width-#notes +1, spectra_y-spectra_height*0.5)
		gl.Vertex(x+width-#notes +1, spectra_y-spectra_height)
		gl.Vertex(x+width-#notes, spectra_y-spectra_height)

	end
	--- draw octave markers
	local octaves=#notes/notes.steps_per_octave
	for i=0,octaves-1 do
		local x=i*notes.steps_per_octave+3*halftone_width
		gl.Color(0,0,0)
		gl.Vertex(x+width-#notes, piano_keyboard_y)
		gl.Vertex(x+width-#notes, piano_keyboard_y-piano_keyboard_height)
		gl.Color(0,0,0.5)
		gl.Vertex(x+width-#notes +notes.steps_per_octave, piano_keyboard_y - piano_keyboard_height)
		gl.Vertex(x+width-#notes +notes.steps_per_octave, piano_keyboard_y)
	end

	--- draw 'keyboard' underneath
	for i=0,halftones-1 do
		local is_black=black[((i-3)%12) +1]
		if is_black then
			gl.Color(0,0.1,0)
		else
			gl.Color(0,0.5,0)
		end
		local x=i*halftone_width
		gl.Vertex(x+width-#notes, piano_keyboard_y )
		gl.Vertex(x+width-#notes +halftone_width-1, piano_keyboard_y)
		gl.Vertex(x+width-#notes +halftone_width-1, piano_keyboard_y - piano_keyboard_height)
		gl.Vertex(x+width-#notes, piano_keyboard_y - piano_keyboard_height)
	end

	gl.End();

	return false -- do not override engine's DrawHUD
end


function load_scenefractal_shader(brightness, time, low, high)
	os.setlocale("C")-- fixing some weird bug
	--LogMessage("low:" .. table.tostring(low))
	--LogMessage("high:" .. table.tostring(high))

	mvis_brightness=GetOption("music_visualizer/brightness")+0
	mvis_distortion=GetOption("music_visualizer/distortion")+0
	mvis_filter_melody=GetOption("music_visualizer/filter_melody")+0



	local scl=60.0;

	local scale={scl/(high[1]-low[1]), scl/(high[2]-low[2]), scl/(high[3]-low[3])}
	local translate={-low[1]*scale[1], -low[2]*scale[2], -low[3]*scale[3]}

	local p=10
	local q=3*mvis_distortion
	local pulse_strength=100*mvis_brightness
	local stripe_br=0


	local shader=GetShader("$$data/music_visualizer_2_v.glsl", "$$data/test_f.glsl")
	gl.UseProgram(shader)

	--{brightness*(0.2+sound_color[1]*pulse_strength), brightness*(0.2+sound_color[2]*pulse_strength), brightness*(0.2+sound_color[3]*pulse_strength)}


	gl.SetUniforms(shader, {
		sharpness=0.001, -- +(0.001*(sound_color[1]+sound_color[2]+sound_color[3]))
		brightness={brightness,brightness,brightness},
		stripe_brightness={stripe_br+sound_color[1]*pulse_strength, stripe_br+sound_color[2]*pulse_strength, stripe_br+sound_color[3]*pulse_strength},
		stripe_matrix={
		0, scale[1], 0, 0,
		0, 0, scale[2], 0,
		scale[3], 0, 0, 0,
		translate[1]+rgb_t[1]*p+sound_color[1]*q, translate[2]+rgb_t[2]*p+sound_color[2]*q, translate[3]+rgb_t[3]*p+sound_color[3]*q, 1
		}
	})
end
