LogMessage("running relativistic visualizer")
--- rendering options, you can change it to load other file
dofile(VFSTranslate("$$data/music_visualizers/relativistic_options.lua"))


Script={} ---disable all built in scripts

cam_recording={}
recording_frame=1
do_record_camera_motion=false
playback_terminated=false


if not rendering then -- record the frames when c is pressed, save when c is pressed again
	function Script:RecordFrames()
		while true do
			if do_record_camera_motion then
				local tmp={pos=true, old_pos=true, orientation=true, old_orientation=true}
				GetObjectState(GetMyID(), tmp)
				cam_recording[recording_frame]={
					cur=PosOriToMatrix(tmp.pos, tmp.orientation),
					old=PosOriToMatrix(tmp.old_pos, tmp.old_orientation)
					--,test_stuff=tmp
				}
				recording_frame=recording_frame+1
			end
			Sleep()
		end
	end

	function DrawHUDFunc.DrawCrosshair(width,height,bloom)
		if game_paused then
			return
		end
		if do_record_camera_motion then
			if bloom then
				gl.Color(1,0,0)
			else
				gl.Color(1,0,0)
			end
		else
			if bloom then
				gl.Color(0,1,0)
			else
				gl.Color(1,1,0)
			end
		end
		gl.Begin(gl.LINES)
		local x=width*0.5
		local y=height*0.5
		local s=20 -- cross hair size
		local gs=5 --gap size
		gl.Vertex(x-s,y)
		gl.Vertex(x-gs,y)

		gl.Vertex(x+s,y)
		gl.Vertex(x+gs,y)

		gl.Vertex(x,y-s)
		gl.Vertex(x,y-gs)

		gl.Vertex(x,y+s)
		gl.Vertex(x,y+gs)
		gl.End()
	end

	function WriteRecording()
		local file=io.open (record_to, "w")
		file:write("cam_recording=")
		--local st=table.tostring(cam_recording)
		--file:write(st)
		table.write(cam_recording, file, 0)
		file:close()
		LogMessage("Saved camera path")
	end

	function on_char(c)
		if c=="c" then
			if do_record_camera_motion then
				WriteRecording()
			end
			do_record_camera_motion=not do_record_camera_motion
		end
	end

	function on_exit_game()
		WriteRecording()
	end
else

	function Script:PlaybackFrames()
		dofile(play_from)
		Sleep()
		SetObjectState(GetMyID(), {disable_control=true})
		for k,v in ipairs(cam_recording) do
			Sleep()
			local pos, ori=MatrixToPosOri(v.cur)
			local old_pos, old_ori=MatrixToPosOri(v.old)
			local state={
				pos=pos,
				old_pos=old_pos,
				orientation=ori,
				old_orientation=old_ori
			}
			LogMessage(table.tostring(state))
			LogMessage(table.tostring(v))
			SetObjectState(GetMyID(), state)
			LogMessage("Did set state for frame *********************************************************************")
		end
		Sleep()
		playback_terminated=true
	end
end
--[[
function Script:MoveCamera()
	local start_state={get_all=true}
	GetObjectState(GetMyID(), start_state)
	local cur_state=deepcopy(start_state)
	local game_frame=0
	while true do
		Sleep()
		game_frame=game_frame+1
		cur_state.prev_pos=
	end
end
]]

local speed=0
local true_vel={0,0,0}

function Script:SpawnStartBooster()
	local pos=my_state.pos
	NewGameObject{type="SimpleBonusPod", pos=pos, bonus_id=2, bonus_lifetime=2000}
end

--[[
function Script:Accelerate()
	local t=1
	--SetObjectState(GetMyID(), {orientation={-0.707106781,0.707106781,0,0}})

	while true do
		speed=c*(1.0-1.0/(t*t))
		GetObjectState(GetMyID(), my_state)

		true_vel=MulQuatVec(my_state.orientation,{0, 0, -speed})
		local gamma=1/math.sqrt(1-speed*speed/(c*c))

		SetObjectState(GetMyID(), {vel=ScaleVec(true_vel, gamma) })


		t=t+1.0/70.0
		Sleep()
		--LogMessage("spam");
	end
end
]]

local time=0
local interp_t=0

local override_dt=GetCmdOption("override_dt")+0

local positions={}
local positions_index=0

function Script:PositionRecorder()
	for i=1,3 do
		positions[i]=deepcopy(my_state.pos)
	end
	while true do
		Sleep()
		positions[1]=positions[2]
		positions[2]=positions[3]
		positions[3]=deepcopy(my_state.pos)
	end
end


local do_rendering=false
function Script:StartRendering()
	for i=1,4 do --- skip first 4 updates
		Sleep()
	end
	do_rendering=rendering
end

function on_pre_draw(dt, interpolate_t)
	time=time+dt;
	interp_t=interpolate_t
	--LogMessage(interp_t)
end

local frame_number=0
local reentry=false
function on_screen_frame(dt, interpolate_t)
	if reentry or playback_terminated or not do_rendering then return end
	reentry=true
	if frame_number<nframes then
		if really_rendering then
			--RenderScreenshot(string.format(name_format_string, frame_number), res_x, res_y)
			RenderImage(string.format(left_name_format_string, frame_number), res_x, res_y,
			string.format(left_raw_name_format_string, frame_number),
			string.format(left_bloom_name_format_string, frame_number),
			-0.5*(GetOption("display/stereo/distance")+0), -0.5*(GetOption("display/stereo/shift")+0)
			)

			RenderImage(string.format(right_name_format_string, frame_number), res_x, res_y,
			string.format(right_raw_name_format_string, frame_number),
			string.format(right_bloom_name_format_string, frame_number),
			0.5*(GetOption("display/stereo/distance")+0), 0.5*(GetOption("display/stereo/shift")+0)
			)
		end
		frame_number=frame_number+1
	end
	reentry=false
end

if rendering then
	DisableHUD(2)
end

local aberration_speed=0.0

function override_field_of_view(fov)
	local v1=SubVec(positions[2],positions[1])
	local v2=SubVec(positions[3],positions[2])
	local interpolated_v=AddVec(v1,ScaleVec(SubVec(v2,v1), interp_t))
	interpolated_v=ScaleVec(interpolated_v, 70.0)

	local fvc=ScaleVec(interpolated_v, 1.0/c)
	local v=ScaleVec(fvc, 1.0/math.sqrt(1.0+Len2(fvc)))


	GetObjectState(GetMyID(), my_state)

	local local_vel=MulQuatVec({-my_state.vis_orientation[1], my_state.vis_orientation[2],my_state.vis_orientation[3],my_state.vis_orientation[4]}, v)

	aberration_speed=-local_vel[3]

	if aberration_speed<1E-10 then
		return fov
	end

	local result=2*math.acos((math.cos(fov/2)+aberration_speed)/(1+math.cos(fov/2)*aberration_speed))
	--LogMessage(result, fov)
	return result*zoom_compensation_factor + fov*(1.0-zoom_compensation_factor)
end

function load_scenefractal_shader(brightness, time, low, high)
	local v1=SubVec(positions[2],positions[1])
	local v2=SubVec(positions[3],positions[2])
	local interpolated_v=AddVec(v1,ScaleVec(SubVec(v2,v1), interp_t))
	interpolated_v=ScaleVec(interpolated_v, 70.0)

	local s=GetShader("$$data/srt_test_v.glsl", "$$data/test_f.glsl")
	gl.UseProgram(s)

	--[[
fake velocity to true velocity:
fv=v*1/sqrt(1-v.v)
true velocity from fake:
fv.fv=v.v*1/(1-v.v)
a=b/(1-b)
a-ab=b
a/b - a = 1
a/b = a+1
b=a/(a+1)
v.v=fv.fv/(fv.fv+1)
vectors are parallel
v=fv/(fv.fv+1)

]]
	local fvc=ScaleVec(interpolated_v, 1.0/c)
	local v=ScaleVec(fvc, 1.0/math.sqrt(1.0+Len2(fvc)))


	GetObjectState(GetMyID(), my_state)

	local local_vel=MulQuatVec({-my_state.vis_orientation[1], my_state.vis_orientation[2],my_state.vis_orientation[3],my_state.vis_orientation[4]}, v)

	--aberration_speed=-local_vel[3]

	gl.SetUniforms(s, {
	brightness=brightness,
	vel=local_vel
	})
end
